GorLib = GorLib or {}

local DAY, HOUR, MINUTE = 86400, 3600, 60
local DAYISH, HOURISH, MINUTEISH, SOONISH = 3600 * 23.5, 60 * 59.5, 59.5, 5.5
local HALFDAYISH, HALFHOURISH, HALFMINUTEISH = DAY/2 + 0.5, HOUR/2 + 0.5, MINUTE/2 + 0.5

local round = function(x) return floor(x + 0.5) end

GorLib.GetTimeText =  function(remain)
	if remain < MINUTEISH then
		local seconds = (remain)
		return (seconds == 0 and '') or seconds
	elseif remain < HOURISH then
		return '%dm', (remain/MINUTE)
	elseif remain < DAYISH then
		return '%dh', (remain/HOUR)
	else
		return '%dd', (remain/DAY)
	end
end

GorLib.Loader = function(event, Func)
	local f = CreateFrame("Frame")
	
	f.ChangeFunc = function(newFunc)
		f:SetScript("OnEvent", function(self, event)
			newFunc(self, event)
		end)
	end
	
	f.RemoveEvent = function(removeEvent)
		f:UnregisterEvent(removeEvent)
	end
	
	f.AddEvent = function(newEvent)
		f:RegisterEvent(newEvent)
	end
	
	f:RegisterEvent(event)
	f:SetScript("OnEvent", function(self, event)
		Func(self, event)
	end)
	return f
end

GorLib.Container = function(name, sets)
	local f = CreateFrame("Frame", name)

	f:EnableMouse(false)
	function f.OnDragStop(self) self:StopMovingOrSizing()   end
	function f.OnDragStart(self) f:StartMoving() GorLib.SavePos(f, sets) end
	local handlers = {"OnDragStart", "OnDragStop"}
	f:RegisterForDrag("LeftButton")
	for _, handler in ipairs(handlers) do
  		  f:SetScript(handler, function(self, button) self[handler](self) end)
		end
	f:SetFrameLevel(5)

	f.texture = f:CreateTexture(nil, 'ARTWORK')
	f.texture:SetTexture(0, 1, 0, .25)

	f:SetScript("OnEnter", function() f.texture:SetTexture(0, 1, .5, .25)  end)
	f:SetScript("OnLeave", function() f.texture:SetTexture(0, 1, 0, .25) end)

	f.texture:Hide()
	f.texture:SetAllPoints(f)
	f:SetClampedToScreen(true)
	function f.ConfigMode()
		if not f.configmode then
			f.configmode = 1
			f:SetMovable(true)
			f.texture:Show()
			f:EnableMouse(true)
		else
			f.configmode = nil
			f:SetMovable(false)
			f.texture:Hide()
			f:EnableMouse(false)
		end
	end
	return f
end

function GorLib.GetChildren(parent, table, max)
	parent.Kids = {}
	for i, b in pairs(table) do
		tinsert(parent.Kids, b)
	end
	for i, b in pairs(parent.Kids) do
		if max and i > max then
			tremove(parent.Kids, i)
		end
	end 
end

GorLib.BasicButtonBarOptions = function(name, parent, kids, SETTINGS, LOCAL, config)
	local f = GorLib.NewMenu(name.." Options", UIParent, name.." Options")
		f:SetScript("OnShow", function(self) self:ClearAllPoints()self:SetPoint("Center", 0, 0)end)
		f:SetSize(218, 120)

		do
			local function LayoutSlider(title, low, high, vari)
				local s = GorLib.NewSlider(title, f, low, high, 1)
				s.OnValueChanged = function(self, value)
					if vari == "scale" then
						GorLib.SetFrameScale(parent, SETTINGS.layout, value/100)
					else
						SETTINGS.layout[vari] = value
						GorLib.Layout(parent, SETTINGS.layout, kids)
					end
				end
				s.OnShow = function(self)
					if vari == "scale" then
						self:SetValue(SETTINGS.layout[vari] * 100)
					else
						self:SetValue(SETTINGS.layout[vari])
					end
				end
				
				s.OnShow(s)
				return s
			end
			
			if not( #kids < 2 ) then
				f.pad = LayoutSlider("Padding", -16, 32,  "pad")
				f.space = LayoutSlider("Spacing", -16, 32,  "space")
				f.cols = LayoutSlider("Columns",  1,  #kids,   "columns")
			end
			f.scale = LayoutSlider("Scale", 	50,  150, "scale")	
	
			local function Check(title, option, vari)
				local c = GorLib.NewCheckButton(title, f)
				c.OnShow = function(self)
					self:SetChecked(SETTINGS[option][vari])
				end
				c.OnClick = function(self)
					SETTINGS[option][vari] = self:GetChecked()
					GorLib.Layout(parent, SETTINGS.layout, kids)
				end
				
				c.OnShow(c)
				return c
			end
	
			if not( #kids < 2 ) then
			
				f:SetHeight(244)
			
				f.LTR = Check("Left to Right", "layout", "LtR")
				f.LTR:SetPoint("Topleft", 20, -60)
				
				f.TTB = Check("Top to Bottom", "layout", "TtB")
			end
			
		
			local reset = CreateFrame("Button", f:GetName().."Reset", f, "UIMenuButtonStretchTemplate")
			reset:SetPoint("TopRight", -10, -35)
			reset:SetWidth(70)
			reset:SetText("Reset")
			reset:SetScript("OnClick", function()
				if LOCAL.Reset then
					LOCAL:Reset()
				end
			end)

		if parent.ConfigMode then
			local Config = CreateFrame("Button", f:GetName().."Config", f, "UIMenuButtonStretchTemplate")
			Config:SetPoint("TOP", 0, -35)
			Config:SetWidth(70)
			Config:SetText("Config")

			Config:SetScript("OnClick", function()
				if parent.ConfigMode then
					parent:ConfigMode()
				end
			end)
		end
	
		local button = CreateFrame("Button", f:GetName().."Point", f, "UIMenuButtonStretchTemplate")
		button:SetPoint("TopLeft", 10, -35)
		button:SetWidth(70)
		button:SetText("Position")

		local f =  GorLib.NewMenu(f:GetName().."OptionsPosition", f, name.." Position")
		f:SetSize(260, 160)
		f:SetScript("OnShow", function()
			f:SetPoint("Left", f:GetParent(), "Right")
		end)
			button:SetScript("OnClick", function() ToggleFrame(f) end)
			local function ValueChange(value, val)
				SETTINGS.position[value] = val
				GorLib.Repos(parent, GorLib.GetRelPos(parent))
			end

			--Menus
			local function NewMenu(name, table, value)	
				local function OnShow(self)
					UIDropDownMenu_SetText(self, SETTINGS.position[value])
				end
				local f = GorLib.NewDropMenu(f, name, table, SETTINGS.position[value], nil, OnShow)
				f.OnClick = function(val)
					ValueChange(value, val)
				end
					OnShow(f)
				
				return f
			end
			
			local g = NewMenu("Relative", {
								["Top"]			= "Top", 			-- [1]
								["TopLeft"]		= "TopLeft",		-- [2]
								["TopRight"]	= "TopRight",		-- [3]
								["Bottom"]		= "Bottom",		-- [4]
								["BottomLeft"]	= "BottomLeft",	-- [5]
								["BottomRight"]	= "BottomRight",	-- [6]
								["Left"]		= "Left",		-- [7]
								["Right"]		= "Right"	,	-- [8]
								["Center"]		= "Center",		-- [9]
								},
					"point")
			g:ClearAllPoints()
			g:SetPoint("BottomLeft", 5, 80)
			UIDropDownMenu_SetWidth(g, 200, 0)
			UIDropDownMenu_SetButtonWidth(g, 200) 
			
			local function NewSlider(value, MinMax)
				local slider = GorLib.NewSlider(value.." Offset", f, -MinMax, MinMax, 1)
				local value = string.lower(value)
				slider:SetWidth(170)
				function slider.OnValueChanged(self, val)
					ValueChange(value, val)
				end
				function slider.OnShow(self)
					slider:SetValue(SETTINGS.position[value])
				end
				
				 slider.OnShow(slider)
				return slider
			end

			f.x = NewSlider("X", floor((UIParent:GetWidth()/UIParent:GetScale())/2));
			f.y = NewSlider("Y", floor((UIParent:GetHeight()/UIParent:GetScale())/2) );
			
		end
	return f
end

GorLib.BasicSettings = function(LOCAL, ADDON_NAME, SAVED)
	if not LOCAL.Contain then
		LOCAL.Contain = GorLib.Container(ADDON_NAME, SAVED.position)
	end
	GorLib.Layout(LOCAL.Contain, SAVED.layout, LOCAL.Kids)
	GorLib.Repos(LOCAL.Contain, SAVED.position)
	LOCAL.Menu = GorLib.BasicButtonBarOptions(ADDON_NAME, LOCAL.Contain, LOCAL.Kids, SAVED, LOCAL)
	GorLib.Rescale(LOCAL.Contain,  SAVED.layout.scale)
end

local SelectTab = function(tab)
	tab.selected = 1
	tab.menu:Show()
	tab.Highlight:Show()
	tab.Selected:Show()
end

local DeselectTab = function(tab)
	tab.selected = nil
	tab.menu:Hide()
	tab.Highlight:Hide()
	tab.Selected:Hide()
end

GorLib.CustomTabButton = function(parent, name, title, Selected)
	local t
	if not _G[parent:GetName()..name] then
		t = CreateFrame("Button", parent:GetName()..name, parent)

		t:SetHeight(17)
		t.text = t:CreateFontString(t:GetName().."Text", "OVERLAY", "GameFontNormal")
		t.text:SetPoint("BottomRight")
		t.text:SetPoint("BottomLeft")
		t.text:SetText(title)
	
		t.Border = t:CreateTexture(nil, "BORDER")
		t.Border:SetTexture("Interface/LFGFRAME/LFGBorder")
		t.Border:SetAllPoints(t)
		t.Border:SetTexCoord(.53, .72, .3, 1)
		
		t.Highlight = t:CreateTexture(nil, "ARTWORK")
		t.Highlight:SetTexture("Interface/BUTTONS/UI-Quickslot-Depress")
		t.Highlight:SetPoint("BottomLeft")
		t.Highlight:SetPoint("TopRight", 0, 0)
		t.Highlight:SetTexCoord(0, 1, 0, .7)
		t.Highlight:Hide()
	

		t.Selected = t:CreateTexture(nil, "BACKGROUND")
		t.Selected:SetTexture("Interface/CHATFRAME/UI-ChatIcon-BlinkHilight")
		t.Selected:SetPoint("BottomLeft")
		t.Selected:SetPoint("TopRight", 0, 0)
		t.Selected:SetTexCoord(0, 1, 0, .5)
	
		t:SetScript("OnEnter", function(self) self.Highlight:Show() end)
		t:SetScript("OnLeave", function(self) self.Highlight:Hide() end)
	
		t.menu = CreateFrame("Frame", t:GetName().."Menu", parent, "InsetFrameTemplate2")
		t.menu:SetPoint("TopLeft",    parent, 0, -15)
		t.menu:SetPoint("BottomRight", parent, 0,   0)
	
		if Selected == nil then
			t.menu:Hide()
			t.Selected:Hide()
		end
	
		if not parent.tabs then
			parent.tabs = {}
		end
		tinsert(parent.tabs, t)
	
		t:SetScript("OnClick", function(self)
			for i = 1, 10 do
				DeselectTab(parent.tabs[i])
			end
			SelectTab(self)
		end)
	else
		t = _G[parent:GetName()..name]
	end
	return t
end
